function DrawLine(ctx, x1, y1, x2, y2) {
	ctx.beginPath();
	ctx.moveTo(x1, y1);
	ctx.lineTo(x2, y2);
	ctx.stroke();
}

function DrawPolygon(ctx, fill, stroke, close, poly) {
	if(poly.length < 2) { return false; }
	ctx.beginPath();
	var lastbegin = poly[0]
	ctx.moveTo(lastbegin[0], lastbegin[1]);
	//console.log(poly.length);
	for(var i = 1; i < poly.length; i++) {
		//console.log("i=",i)
		if(poly[i] === null) {
			if(close === true) { ctx.lineTo(lastbegin[0], lastbegin[1]); }
			if(fill === true) { ctx.fill(); } 
			if(stroke === true) { ctx.stroke(); }
			if(i + 1 < poly.length) {
				
				ctx.beginPath();
				i++;
				//console.log("new start: ",i);
				lastbegin = poly[i];
				ctx.moveTo(lastbegin[0], lastbegin[1]);
				
			}
		}
		if(poly[i] !== null && typeof poly[i] == "object") {
			ctx.lineTo(poly[i][0], poly[i][1]);
		}
	}
	if(close === true) { ctx.lineTo(lastbegin[0], lastbegin[1]); }
	if(fill === true) { ctx.fill(); } 
	if(stroke === true) { ctx.stroke(); }
}
	

function DrawPoint(ctx, shape, filled, x, y, w, h) {
	if(h === undefined) { h = w; }
	var wh = w/2;
	var hh = h/2;

	switch(shape) {
		case "$": 
			DrawPolygon(ctx, filled, true, true, [[x, y - hh], [x + wh, y], [x, y + hh], [x - wh, y]]);
			break;
		case "#":
			DrawPolygon(ctx, filled, true, true, [[x - wh, y - hh], [x + wh, y - hh], [x + wh, y + hh], [x - wh, y + hh]]);
			break;
		case "^":
			DrawPolygon(ctx, filled, true, true, [[x - wh, y + hh], [x, y - hh], [x + wh, y + hh]]);
			break;
		case "v": 
			DrawPolygon(ctx, filled, true, true, [[x + wh, y - hh], [x, y + hh], [x - wh, y - hh]]);
			break;
		case "x":
			DrawPolygon(ctx, filled, true, true, [[x - wh, y - hh], [x + wh, y + hh], null, [x + wh, y - hh], [x - wh, y + hh]]);
			break;
		case "*":
			DrawPolygon(ctx, filled, true, true, [[x - wh, y - hh], [x + wh, y + hh], null, [x + wh, y - hh], [x - wh, y + hh], null, [x, y], [x, y - hh]]);
			break;
		case "+": 
			DrawPolygon(ctx, filled, true, true, [[x, y - hh], [x, y + hh], null, [x - wh, y], [x + wh, y]]);
			break;
		case "|": 
			DrawPolygon(ctx, filled, true, true, [[x, y - hh], [x, y + hh]]);
			break;
		case "-": 
			DrawPolygon(ctx, filled, true, true, [[x - wh, y], [x + wh, y]]);
			break;
		case "o": 
			DrawEllipse(ctx, filled, true, x, y, w, h);
			break;
	}
}



function DrawEllipse(ctx, fill, stroke, aX, aY, aWidth, aHeight){
	var hB = (aWidth / 2) * .5522848,
			vB = (aHeight / 2) * .5522848,
			eX = aX + aWidth,
			eY = aY + aHeight,
			mX = aX + aWidth / 2,
			mY = aY + aHeight / 2;
	ctx.moveTo(aX, mY);
	ctx.bezierCurveTo(aX, mY - vB, mX - hB, aY, mX, aY);
	ctx.bezierCurveTo(mX + hB, aY, eX, mY - vB, eX, mY);
	ctx.bezierCurveTo(eX, mY + vB, mX + hB, eY, mX, eY);
	ctx.bezierCurveTo(mX - hB, eY, aX, mY + vB, aX, mY);
	ctx.closePath();
	if(fill === true) { ctx.fill(); } 
	if(stroke === true) { ctx.stroke(); }
}

function DrawDashedLine(ctx, pat, x1, y1, x2, y2) {
	var d = { "x" : x2 - x1, "y" : y2 - y1 };

	var len = Math.sqrt(d.x * d.x + d.y * d.y);
	var pixellen = { "x" : d.x / len, "y" : d.y / len };
	
	var lw = ctx.lineWidth;
	if(lw != 1) {
		for(var i = 0; i < pat.length; i++) {
			pat[i] *= lw;
		}
	}
	
	var pos = 0;
	var i = 0;
	
	/*if(orientation > 0) {
		var patlen = 0;
		for(var i = 0; i < pat.length; i++) {
			patlen += pat[i];
		}
		
		var overlap = len - Math.floor(len / patlen) * patlen;
		if(orientation == 1) { overlap /= 2; }
		console.log("Overlap: ", overlap);
		while(overlap > 0) {
			i %= pat.length;
			overlap -= pat[i];
			i++;
		}
		//console.log("compensated overlap: ", overlap, "next pattern: ", i);
		if(overlap < 0) {
			i += pat.length - 1;
			i %= pat.length; //das vorherige Pattern
			//console.log("checking pattern ", i);
			if(i % 2 == 0) {
				//Zeichnen
				overlap *= -1; //die zu viel abgezogene Strecke muss gezeichnet werden
				DrawLine(ctx, x1 + pixellen.x * pos, y1 + pixellen.y * pos, x1 + pixellen.x * (pos + overlap), y1 + pixellen.y * (pos + overlap));
			}
			pos += overlap;
			i++;
		}
		
	}*/
	var tpat = 0;
	while(pos < len) {
		pos += tpat;
		i %= pat.length;
		tpat = pat[i];
		
		if((len - pos) < tpat) { tpat = len - pos; }
		
		if(i % 2 == 0) {
			DrawLine(ctx, x1 + pixellen.x * pos, y1 + pixellen.y * pos, x1 + pixellen.x * (pos + tpat), y1 + pixellen.y * (pos + tpat));
		}
		
		i++;
	}
}

function hsvToRgb(h, s, v) {
var r, g, b;
var i;
var f, p, q, t;
 
// Make sure our arguments stay in-range
h = Math.max(0, Math.min(360, h));
s = Math.max(0, Math.min(100, s));
v = Math.max(0, Math.min(100, v));
 
// We accept saturation and value arguments from 0 to 100 because that's
// how Photoshop represents those values. Internally, however, the
// saturation and value are calculated from a range of 0 to 1. We make
// That conversion here.
s /= 100;
v /= 100;
 
if(s == 0) {
// Achromatic (grey)
r = g = b = v;
return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
}
 
h /= 60; // sector 0 to 5
i = Math.floor(h);
f = h - i; // factorial part of h
p = v * (1 - s);
q = v * (1 - s * f);
t = v * (1 - s * (1 - f));
 
switch(i) {
case 0:
r = v;
g = t;
b = p;
break;
 
case 1:
r = q;
g = v;
b = p;
break;
 
case 2:
r = p;
g = v;
b = t;
break;
 
case 3:
r = p;
g = q;
b = v;
break;
 
case 4:
r = t;
g = p;
b = v;
break;
 
default: // case 5:
r = v;
g = p;
b = q;
}
 
return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
}